#include <Adafruit_NeoPixel.h>
#define TINY_GSM_MODEM_ESP8266

// Template ID, Device Name and Auth Token are provided by the Blynk.Cloud
#define BLYNK_TEMPLATE_ID "<enter_template_id>"
#define BLYNK_DEVICE_NAME "<enter_device_name>"
#define BLYNK_AUTH_TOKEN "<enter_token>"
#define BLYNK_MAX_READBYTES 1024

// Comment this out to disable prints and save space
#define BLYNK_PRINT SerialUSB

#include <ESP8266_Lib.h>
#include <BlynkSimpleShieldEsp8266.h>

const char auth[] = BLYNK_AUTH_TOKEN;

// My WiFi credentials.
const char ssid[] = "";
const char pass[] = "";

// Create Wifi client.
#define EspSerial Serial1
#define ESP8266_BAUD 115200
ESP8266 wifi(&EspSerial);

//Define the pin values of the input sensors to use from the circuit.
#define CPLAY_LIGHT A8
#define CPLAY_LEFTBUTTON 4

//The following definitions are for the neopixel leds on the cpx.
#define LED_PIN    8
#define LED_COUNT 10
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);
unsigned long lightsOn = 0; //Holds the millis() millisecond of when the neopixel led where last turned on so we can turn them off after 10 seconds.

unsigned long last_sent = 0; //Holds the millis() millisecond of when the last webhook was sent to integromat because we can only send one every minute in the free Blynk version.
int last_question = 0; //Holds the value of the question that is being reviewed now.
bool office_hour_started = false; //Will be true if the office hour has started, and false otherwise.
bool data_checked = false; //false if the instructor didn't check the questions order yet, and true otherwise.

bool leftButtonPressed; //Will be true if the left button is pressed and false otherwise.
int buttonState = 0;    // variable for reading the pushbutton status
double light_value = 0; //Holds the light value recieved from the cpx light sensor.
bool on_break = false; //Indicates if there is a break at the moment or not.

// This function will be called every time the Slider Widget in Blynk app writes values to the Virtual Pin V1.
//This virtual pin tells the cpx which question will be reviewed next.
BLYNK_WRITE(V1)
{
   int pin_value = param.asInt(); // get the question number that will be reviewed next.
   if(pin_value && pin_value!=last_question)
   {
    //publish message to the question topic
    last_question = pin_value;
    sendToIntegromat(String("quest:") + pin_value);
   }
}
// This function will be called every time the Button Widget in Blynk app writes values to the Virtual Pin V2.
//This virtual pin tells the cpx that the office hour is about to start.
BLYNK_WRITE(V2)
{
   int pin_value = param.asInt(); // get 1 if the office hour is about to start, 0 otherwise.
   if(pin_value && !office_hour_started)
   {
    //publish message to the instructor topic that the office hour is about to start.
    office_hour_started = true;
    sendMessageToStudents(1);
   }
}
// This function will be called every time a message is sent from the students or from integromat to the istructor.
BLYNK_WRITE(V3)
{
   int pin_value = param.asInt(); // get the message.
   if(pin_value==1)
   {
    //The integromat questions check is done, so we inform the instructor by lighting the neopixel lights in green.
    blink_lights();
   }
   else if(pin_value==2)
   {
    //The instructor got hand claps from the students.
    show_claps();
   }
}
void setup() {
  SerialUSB.begin(9600);
  EspSerial.begin(ESP8266_BAUD);
  delay(10);
  Blynk.begin(auth, wifi, ssid, pass); //Blynk setup.

  pinMode(CPLAY_LEFTBUTTON, INPUT);//Get input from the left button on the CPX.
  
  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(255); // Set BRIGHTNESS to about 1/5 (max = 255)
  

}

void loop() {
  Blynk.run(); //Start Blynk.


  // read the state of the pushbutton value:
  buttonState = digitalRead(CPLAY_LEFTBUTTON);

  // check if the pushbutton is pressed. If it is, the buttonState is HIGH:
  if (buttonState == HIGH and !data_checked){
    //The instructor wants to check which questions should be reviewed so he pushed the left button.
    data_checked = true;
    sendToIntegromat(String("check:1"));
  }

  //Check if the Instructor wants to take a break, which is signaled by flipping the cpx so it is dark and can be identified by the light sensor.
  light_value = analogRead(CPLAY_LIGHT);//get the updated light value from the light sensor on the cpx.
  if(light_value < 10 && !on_break){
    sendMessageToStudents(2);//We go on a break.
    on_break = true;
  }
  else if(light_value > 60 && on_break){
    sendMessageToStudents(3);//The break is over.
    on_break = false;
  }
  
  //Turn off the neopixel lights 10 seconds after they were turned on.
  if(millis() - lightsOn > 10000){
    turnNeopixelsOff();
  }
}

//This function is responsible for sending messages from the intructor to the integromat.
//Only one message can be sent every minute.
void sendToIntegromat(String param){
  if((millis() - last_sent > 70000) || !last_sent){
    //Write to the virtual pin V0 which activates the Blynk webhooks that was set.
    Blynk.virtualWrite(V0, param);
    last_sent = millis(); //Hold the time of the last sent message.
  }
  else{
    Serial.println("not enough time passed");
  }
}

//This function is responsible for lighting the neopixel lights in green according to messages that the instructor recieves.
void blink_lights(){
  for (int i = 0 ; i < LED_COUNT; ++i){
    strip.setPixelColor(i, 0, 1, 0); 
  }
  strip.show();
  
  lightsOn = millis(); //save the time of the neopixles lighting
}

//This function is used to turn off the neopixel leds.
void turnNeopixelsOff(){
  for (int i = 0 ; i < LED_COUNT; ++i){
    strip.setPixelColor(i, 0, 0, 0); 
  }
  strip.show();
  lightsOn = 0;
}

//This function is responsible for sending messages from the instructor to the students.
void sendMessageToStudents(int msg_id){
  if(msg_id==1){
    //The office hour is about to start.
    sendToIntegromat(String("start:1"));
  }
  else if(msg_id==2){
    //There is a break.
    sendToIntegromat(String("break:1"));
  }
  else if(msg_id==3){
    //The break is over.
    sendToIntegromat(String("over:1"));
  }
}

//The instructor got a good feedback from the students so we light the neopixel leds in rainbow like colors.
//Taken from the example sketch: strandtest_wheel
void show_claps(){
  uint16_t i, j;
  uint8_t wait = 20;
  for(j=0; j<256; j++) {
    for(i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel((i+j) & 255));
    }
    strip.show();
    delay(wait);
  }
  lightsOn = millis(); //save the time of the neopixles lighting
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if(WheelPos < 85) {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if(WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}
